/*****************************************************/
/* pushhook.c                                        */
/* -- Program demonstrates using a message hook to   */
/*    activate the default button with the enter     */
/*    key.  To build:                                */
/*        cc -DSTRICT pushhook.c                     */
/*****************************************************/
#include <windows.h>
#include <windowsx.h>

HHOOK   hhook;

HWND    HwndFindDefault(HWND);

BOOL CALLBACK __export
    FDlgProc(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK __export
    LwMsgHook(int, WPARAM, LPMSG);

#ifdef __BORLANDC__
    #pragma argsused
#endif
int PASCAL WinMain(HINSTANCE hins, HINSTANCE hinsPrev,
  LPSTR lsz, int wShow)
/*****************************************************/
/* -- Entry point.                                   */
/*****************************************************/
    {
    if ((hhook = SetWindowsHookEx(WH_MSGFILTER,
        (HOOKPROC)LwMsgHook, hins, GetCurrentTask()))
      == NULL)
        return FALSE;

    DialogBox(hins, "PushHook", NULL, FDlgProc);
    return UnhookWindowsHookEx(hhook);
    }

#ifdef __BORLANDC__
    #pragma argsused
#endif
BOOL CALLBACK __export
FDlgProc(HWND hwnd, UINT wm, WPARAM wParam,
  LPARAM lParam)
/*****************************************************/
/* -- Trivial sample dialog proc.                    */
/*****************************************************/
    {
    switch (wm)
        {
    default:
        break;

    case WM_INITDIALOG:
        return TRUE;

    case WM_COMMAND:
        if (wParam == IDOK || wParam == IDCANCEL)
            {
            EndDialog(hwnd, wParam == IDOK);
            return TRUE;
            }
        break;  /* End case WM_COMMAND. */
        }       /* End switch wm. */

    return FALSE;
    }

LRESULT CALLBACK  __export LwMsgHook(int msgf,
  WPARAM wParam, LPMSG lpmsg)
/*****************************************************/
/* -- Message hook.                                  */
/* -- Intercept keyup and keydown of return key.     */
/*****************************************************/
    {
    HWND    hwndDefault;

    if (msgf != MSGF_DIALOGBOX)
        goto CallHook;  /* Not a dialog message. */

    if (lpmsg->wParam != VK_RETURN)
        goto CallHook;  /* Not the enter key. */

    if (lpmsg->message != WM_KEYDOWN &&
        lpmsg->message != WM_KEYUP)
        goto CallHook;  /* Not a keyboard message. */

    if ((hwndDefault = HwndFindDefault(lpmsg->hwnd)) ==
      NULL)
        goto CallHook;  /* No default push button. */

    /* Turn it into a space bar press for the */
    /* default push button. */
    lpmsg->wParam = VK_SPACE;
    lpmsg->hwnd = hwndDefault;
    if (lpmsg->message == WM_KEYDOWN)
        SetFocus(hwndDefault);

CallHook:
    return CallNextHookEx(hhook, msgf, wParam,
      (LPARAM)lpmsg);
    }

HWND HwndFindDefault(HWND hwnd)
/*****************************************************/
/* -- Find the current default push button.          */
/* -- Can't use DM_GETDEFID since it won't track the */
/*    push button with the focus.                    */
/*****************************************************/
    {
    /* Find the dialog window. */
    while (GetWindowStyle(hwnd) & WS_CHILD)
        hwnd = GetParent(hwnd);

    for (hwnd = GetWindow(hwnd, GW_CHILD);
      hwnd != NULL;
      hwnd = GetWindow(hwnd, GW_HWNDNEXT))
        if ((SendMessage(hwnd, WM_GETDLGCODE, 0, 0) &
            DLGC_BUTTON) &&
          (GetWindowStyle(hwnd) & 0x0000000f) ==
            BS_DEFPUSHBUTTON)
            return hwnd;

    return NULL;
    }
